

set -ex



ffmpeg --help
ffmpeg -codecs | grep "DEVI.S zlib"
test -f $PREFIX/lib/libavcodec.so
test -f $PREFIX/lib/libavdevice.so
test -f $PREFIX/lib/libswresample.so
test -f $PREFIX/lib/libpostproc.so
test -f $PREFIX/lib/libavfilter.so
test -f $PREFIX/lib/libswresample.so
test -f $PREFIX/lib/libavcodec.so
test -f $PREFIX/lib/libavformat.so
test -f $PREFIX/lib/libswscale.so
test -f $PREFIX/lib/libavresample.so
exit 0
